@@ -1,31 +1,34 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
-
 /* gstreamermm - a C++ wrapper for gstreamer
  *
  * Copyright 2008 The gstreamermm Development Team
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include <gtkmm/main.h>
 #include <gstreamermm/init.h>
 #include <gstreamermm/element.h>
-#include <gstreamermm/elementfactory.h>
 #include <gstreamermm/pad.h>
 #include <gstreamermm/pipeline.h>
+
+// Plug-ins
 #include <gstreamermm/filesrc.h>
+#include <gstreamermm/oggdemux.h>
+#include <gstreamermm/vorbisdec.h>
+#include <gstreamermm/audioconvert.h>
+#include <gstreamermm/alsasink.h>
+
 #include <iostream>
 #include "player_window.h"
 
@@ -66,7 +69,7 @@ main (int argc, char *argv[])
   }
 
   // Parses the ogg streams into elementary streams (note that an ogg file may contain a video stream too)
-  Glib::RefPtr<Gst::Element> parser = Gst::ElementFactory::create_element("oggdemux");
+  Glib::RefPtr<Gst::OggDemux> parser = Gst::OggDemux::create();
   if(!parser)
   {
     std::cerr << "oggdemux element could not be created" << std::endl;
@@ -74,7 +77,7 @@ main (int argc, char *argv[])
   }
 
   // Decodes a vorbis stream
-  decoder = Gst::ElementFactory::create_element("vorbisdec");
+  decoder = Gst::VorbisDec::create();
   if(!decoder)
   {
     std::cerr << "vorbisdec element could not be created" << std::endl;
@@ -82,7 +85,7 @@ main (int argc, char *argv[])
   }
 
   // Converts audio to a format which can be used by the next element
-  Glib::RefPtr<Gst::Element> conv = Gst::ElementFactory::create_element("audioconvert");
+  Glib::RefPtr<Gst::AudioConvert> conv = Gst::AudioConvert::create();
   if(!conv)
   {
     std::cerr << "audioconvert element could not be created" << std::endl;
@@ -90,7 +93,7 @@ main (int argc, char *argv[])
   }
 
   // Outputs sound to an ALSA audio device
-  Glib::RefPtr<Gst::Element> sink = Gst::ElementFactory::create_element("alsasink");
+  Glib::RefPtr<Gst::AlsaSink> sink = Gst::AlsaSink::create();
   if(!sink)
   {
     std::cerr << "sink element could not be created" << std::endl;
