@@ -2,24 +2,23 @@
 
 /* generate_plugin_gmmproc_file.cc
  *
- * Copyright (C) 2001 The Free Software Foundation
+ * Copyright 2008 The gstreamermm Development Team
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU General Public
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-
 #include "gst_type_is_a_pointer.h"
 
 #include <gst/gst.h>
@@ -223,23 +222,27 @@ void get_property_wrap_statements(Glib::ustring& wrapStatements,
         if ((G_TYPE_IS_ENUM(propertyGType) || G_TYPE_IS_FLAGS(propertyGType)) &&
           !(enumIsWrapped = is_wrapped_enum(propertyCType)))
         {
+          Glib::ustring propertyCppType = propertyCType.substr(3);
+
           enumWrapStatements += "_WRAP_ENUM(" + propertyCType.substr(3) + ", " +
             propertyCType + ")\n";
           enumWrapStatements += "_CCONVERSION(`" + propertyCType + "',`" +
-            propertyCType.substr(3) + "')dnl\n";
+            propertyCppType + "',`" + propertyCppType + "')dnl\n";
 
           Glib::ustring enumGetTypeFunctionName =
             get_cast_macro(propertyCType).lowercase() + "_get_type";
 
           enumGTypeFunctionDefinitions +=
-            "extern \"C\" GType " + enumGetTypeFunctionName + "()\n" +
+            "static GType " + enumGetTypeFunctionName + "()\n" +
             "{\n" +
-            "  return g_type_from_name(\"" + propertyCType + "\");\n" +
+            "  static GType const type = g_type_from_name(\"" +
+              propertyCType + "\");\n" +
+            "  return type;\n" +
             "}\n\n";
         }
 
         wrapStatements += "  _WRAP_PROPERTY(\"" + propertyName + "\", " +
-          "_CCONVERT(" + propertyCType + ", true))\n";
+          "_CCONVERT(" + propertyCType + ", `ret'))\n";
 
         if (!G_TYPE_IS_ENUM(propertyGType) || enumIsWrapped)
           includeMacroCalls += "_CCONVERSION_INCLUDE(" + propertyCType + ")dnl\n";
@@ -310,7 +313,7 @@ void get_signal_wrap_statements(Glib::ustring& wrapStatements,
       includeMacroCalls += "_CCONVERSION_INCLUDE(" + returnCType + ")dnl\n";
 
       wrapStatement = "  _WRAP_SIGNAL(_CCONVERT("  + returnCType +
-        ", true) " + signalMethodName + "(";
+        ", `ret') " + signalMethodName + "(";
 
       cClassSignalDeclarations += "  " + returnCType + " (*" +
         signalMethodName + ") (" + cTypeName + "* element";
@@ -338,12 +341,13 @@ void get_signal_wrap_statements(Glib::ustring& wrapStatements,
           if (gst_type_is_a_pointer(paramGType))
           {
             convertMacros += "#m4 _CONVERSION(``" + paramCType +
-              "'', _LQ()_CCONVERT(" + paramCType + ")_RQ(), ";
+              "'', _LQ()_CCONVERT(" + paramCType + ",`param')_RQ(), ";
             convertMacros += g_type_is_a(paramGType, GST_TYPE_MINI_OBJECT) ?
               "``Gst::wrap($3, true)'')\n" : "``Glib::wrap($3, true)'')\n";
           }
 
-          wrapStatement += "_CCONVERT(" + paramCType + ") " + paramName;
+          wrapStatement += "_CCONVERT(" + paramCType + ", `param') " +
+            paramName;
 
           cClassSignalDeclarations += ", " + paramCType + " " + paramName;
 
@@ -367,8 +371,41 @@ void get_signal_wrap_statements(Glib::ustring& wrapStatements,
     g_type_default_interface_unref(ginterface_ref); // for interface ref above.
 }
 
+void get_interface_macros(Glib::ustring& interfaceMacros,
+  Glib::ustring& includeMacroCalls,
+  Glib::ustring& cppExtends)
+{
+  guint n_interfaces = 0;
+  GType* interfaces = g_type_interfaces(type, &n_interfaces);
+
+  GType parent_type = g_type_parent(type);
+
+  for (int i = 0; i < n_interfaces; i++)
+  {
+    if (!g_type_is_a(parent_type, interfaces[i]))
+    {
+      Glib::ustring  interfaceCType = g_type_name(interfaces[i]) +
+        (Glib::ustring) "*";
+
+      cppExtends += "public _CCONVERT(`" + interfaceCType + "',`type')";
+
+      if (i < n_interfaces - 1)
+        cppExtends += ", ";
+
+      interfaceMacros += "  _IMPLEMENTS_INTERFACE(_CCONVERT(`" +
+        interfaceCType + "',`type'))\n";
+
+      includeMacroCalls += "_CCONVERSION_INCLUDE(" + interfaceCType + ")dnl\n";
+    }
+  }
+
+  g_free(interfaces);
+}
+
 void generate_hg_file(const Glib::ustring& includeMacroCalls,
   const Glib::ustring& enumWrapStatements,
+  const Glib::ustring& cppExtends,
+  const Glib::ustring& interfaceMacros,
   const Glib::ustring& propertyWrapStatements,
   const Glib::ustring& signalWrapStatements)
 {
@@ -387,23 +424,33 @@ void generate_hg_file(const Glib::ustring& includeMacroCalls,
 
   std::cout << enumWrapStatements << std::endl;
 
-  std::cout << "/** " << nmspace << "::" << cppTypeName << " — " << pluginName << " plugin." << std::endl;
+  std::cout << "/** " << nmspace << "::" << cppTypeName << " - " << pluginName << " plugin." << std::endl;
   std::cout << " * Please include <" << target << "/" <<
-    cppTypeName.lowercase() << ".h> to use.  Also, though using the\n"
-    " * underlying GObject is fine, using its C <B>type</B> is not "
-    "guaranteed to be\n * API stable across releases because it is not "
-    "guaranteed to always remain\n * the same." << std::endl;
+    cppTypeName.lowercase() << ".h> to use.  Please note that, though\n"
+    " * using the underlying GObject is fine, using its C <B>type</B> is not\n"
+    " * guaranteed to be API stable across releases because it is not "
+    "guaranteed to\n * always remain the same.  Also, not all plug-ins are "
+    "available on all systems\n * and the ones that aren't available are not "
+    "included in the build." << std::endl;
   std::cout << " *" << std::endl;
   std::cout << " * @ingroup " << nmspace << "Plugins" << std::endl;
   std::cout << " */" << std::endl;
   std::cout << "class " << cppTypeName << std::endl;
-  std::cout << ": public " << parentNameSpace << "::" << cppParentTypeName <<
-    std::endl;
+  std::cout << ": public " << parentNameSpace << "::" << cppParentTypeName;
+
+  if (!cppExtends.empty())
+    std::cout << ", " << cppExtends;
+  
+  std::cout << std::endl;
+
   std::cout << "{" << std::endl;
   std::cout << "  _CLASS_GOBJECT(" << cppTypeName << ", " << cTypeName <<
     ", " << castMacro << ", " << parentNameSpace << "::" <<
-    cppParentTypeName << ", " << cParentTypeName << ")" << std::endl <<
-    std::endl;
+    cppParentTypeName << ", " << cParentTypeName << ")" << std::endl;
+
+  std::cout << interfaceMacros << std::endl;
+
+  std::cout << "  _IS_GSTREAMERMM_PLUGIN" << std::endl << std::endl;
 
   std::cout << "protected:" << std::endl;
   std::cout << "  " << cppTypeName << "();" << std::endl;
@@ -446,7 +493,9 @@ void generate_ccg_file(const Glib::ustring& enumGTypeFunctionDefinitions,
 
   Glib::ustring getTypeName = castMacro.lowercase() + "_get_type";
 
-  std::cout << "extern \"C\" GType " << getTypeName << "();" << std::endl;
+  std::cout << "extern \"C\"" << std::endl;
+  std::cout << "{" << std::endl << std::endl;
+
   std::cout << "GType " << getTypeName << "()" << std::endl;
   std::cout << "{" << std::endl;
   std::cout << "  static GType type = 0;" << std::endl;
@@ -474,6 +523,8 @@ void generate_ccg_file(const Glib::ustring& enumGTypeFunctionDefinitions,
 
   std::cout << enumGTypeFunctionDefinitions;
 
+  std::cout << "} // extern \"C\"" << std::endl << std::endl;
+
   std::cout << "namespace " << nmspace << std::endl;
   std::cout << "{" << std::endl << std::endl;
 
@@ -483,7 +534,7 @@ void generate_ccg_file(const Glib::ustring& enumGTypeFunctionDefinitions,
 
   std::cout << cppTypeName << "::" << cppTypeName << "(const Glib::ustring& name)" << std::endl;
   std::cout << ": _CONSTRUCT(\"name\", name.c_str())" << std::endl;
-  std::cout << "{}" << std::endl;
+  std::cout << "{}" << std::endl << std::endl;
 
   std::cout << std::endl << "}" << std::endl;
 }
@@ -594,22 +645,8 @@ int main(int argc, char* argv[])
     cppParentTypeName = cParentTypeName.substr(3);
     castMacro = get_cast_macro(cTypeName);
 
-    if (cppParentTypeName.compare("BaseSrc") == 0 ||
-      cppParentTypeName.compare("BaseSink") == 0 ||
-      cppParentTypeName.compare("BaseTransform") == 0 ||
-      cppParentTypeName.compare("PushSrc") == 0 ||
-      cppParentTypeName.compare("Pipeline") == 0 ||
-      cppParentTypeName.compare("Bin") == 0 ||
-      cppParentTypeName.compare("Element") == 0)
-    {
-      parentInclude = "gstreamermm";
-      parentNameSpace = "Gst";
-    }
-    else
-    {
-      parentInclude = target;
-      parentNameSpace = nmspace;
-    }
+    parentInclude = target;
+    parentNameSpace = nmspace;
 
     if (hgFile || ccgFile)
     {
@@ -628,8 +665,14 @@ int main(int argc, char* argv[])
 
       if (hgFile)
       {
-        generate_hg_file(includeMacroCalls, enumWrapStatements,
-          propertyWrapStatements, signalWrapStatements);
+        Glib::ustring interfaceMacros;
+        Glib::ustring cppExtends;
+
+        get_interface_macros(interfaceMacros, includeMacroCalls,
+          cppExtends);
+
+        generate_hg_file(includeMacroCalls, enumWrapStatements, cppExtends,
+          interfaceMacros, propertyWrapStatements, signalWrapStatements);
       }
       else
       {
